; Utils module
;
; NOTE: some code comes from Blarg's tutorial and the nes wiki
;
.segment "CODE_UTIL"

;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=--=-=-=-=-
; Include some useful symbolic constants
;
.include "./src/inc/NES.inc"	

;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=--=-=-=-=-
; What can be viewed by other modules
;
.export subInitNes
.export subLoadPalette
.export subUpdateSprRam
.export subClearNameTables
.export subSetScroll
.export subReadJoy1
.export subWaitNmi

;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=--=-=-=-=-
; Import some zero pages variables used
; as parameters
;
.importzp zpJoy1Status
.importzp zpCopySrcAdr
.importzp zpNmiCount


;->BEGIN----------------------------------------------------------------------
; Initializes NES
;
.proc subInitNes
	sei          ; ignore IRQs
	cld          ; disable decimal mode
	
	lda #0 
	sta PPU_CTRL ; disable NMI
	sta PPU_MASK ; disable rendering

	; Now clear the ram
	lda #0
	ldx #0
	
@clrmem:
	sta $000,x ; Zero page
;	sta $100,x ; No need to clear the stack, would kill the return address.
	sta $200,x ; Ram
	sta $300,x
	sta $400,x
	sta $500,x
	sta $600,x
	sta $700,x ; Remove this if you're storing reset-persistent data
	
	inx
	bne @clrmem

	; wait 2 vblank for PPU to stabilize
	JSR subWaitVBlank
	jsr subWaitVBlank

        ; Finally, clear all name tables
        jsr subClearNameTables
        
	rts
.endproc
;-<END------------------------------------------------------------------------


;->BEGIN----------------------------------------------------------------------
; Loads palette
;
; Uses: zpCopySrcAdr for the address of the palette
;
.proc subLoadPalette
	lda #$3F
	sta PPU_ADDR
	lda #0
	sta PPU_ADDR
	
	ldy #0
@load:  lda (zpCopySrcAdr),y
	sta PPU_DATA
	iny
	cpy #32
	bne @load
	
	rts
.endproc	
;-<END------------------------------------------------------------------------


;->BEGIN----------------------------------------------------------------------
; Copy OAM data to SPR-RAM
;
; Note: Not sure if it's that useful or not since you require to call the
;       function for a few lines.. In some non intensive case, it may be ok.
;
.proc subUpdateSprRam
	LDA #<OAM
	STA PPU_OAM_ADDR ; set the low byte (00) of the RAM address
	LDA #>OAM
	STA SPR_DMA
	
	rts
.endproc	
;-<END------------------------------------------------------------------------


;->BEGIN----------------------------------------------------------------------
; Clears all four nametables
;
.proc subClearNameTables
	lda #$20
	sta PPU_ADDR
	lda #$00
	sta PPU_ADDR
	
	ldx #32*4
@clear_nt:
	txa
	pha
	
	lda #0
	ldx #32
@clear_row:
	sta PPU_DATA
	dex
	bne @clear_row
	
	pla
	tax
	dex
	bne @clear_nt
	
	rts
.endproc	
;-<END------------------------------------------------------------------------


;->BEGIN----------------------------------------------------------------------	
; Scroll to X/Y
; Note : Wait for NMI is done in the main randering loop
; Note2: It is a little bit overkill for now? (just for moving.. It could be done
;        in the main)
;
.proc subSetScroll
	lda PPU_STATUS    ; Reset latch
	
	; Set X and Y scroll position
	stx PPU_SCROLL
	sty PPU_SCROLL
	
	rts
.endproc	
;-<END------------------------------------------------------------------------


;->BEGIN----------------------------------------------------------------------
; Reads controller 1 into A
; Preserved: X, Y
;
.proc subReadJoy1
	; Strobe controller to reset its shift register
	lda #1
	sta JOYPAD_PORT1
	lda #0
	sta JOYPAD_PORT1
	
	; Set high bit of variable we'll accumulate bits in
	lda #$80
	sta zpJoy1Status
	
	; Read the 8 button bits
@loop:  ; Read next button into lowest bit of A
	lda JOYPAD_PORT1
	
	; Shift it into carry bit, then rotate carry into high
	; bit of temp. Lowest bit of temp ends up in carry.
	lsr a
	ror zpJoy1Status
	
	; Once carry is set, the loop has executed 8 times
	bcc @loop
	
	; Return accumulated bits
	lda zpJoy1Status
	rts
.endproc	
;-<END------------------------------------------------------------------------


;->BEGIN----------------------------------------------------------------------
; Waits for nmi_count to change
; Preserved: X, Y
;
.proc subWaitNmi
	lda zpNmiCount
	
@wait:  cmp zpNmiCount
	beq @wait

	rts
.endproc	
;-<END------------------------------------------------------------------------

;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= Not used -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

;->BEGIN----------------------------------------------------------------------
; Turns NMI on
;
; Note: Maybe not that useful in this state
;
.proc subSetNmiOn
	jsr subWaitVBlank
	
	lda #PPU_CTRL_NMI
	sta PPU_CTRL
	
	rts
.endproc	
;-<END------------------------------------------------------------------------


;->BEGIN----------------------------------------------------------------------
; Waits for VBL
; Preserved: A, X, Y 
;
.proc subWaitVBlank
	; Clear VBL bit in case it was already set and
	; VBL was just about to end
	bit PPU_STATUS

@wait:  bit PPU_STATUS
	bpl @wait
	rts
.endproc
;-<END------------------------------------------------------------------------